@@ -5,6 +5,7 @@ from contextlib import contextmanager
 from pathlib import Path
 from pyln.client import RpcError
 from pyln.testing.btcproxy import BitcoinRpcProxy
+from pyln.testing.gossip import GossipStore
 from collections import OrderedDict
 from decimal import Decimal
 from ephemeral_port_reserve import reserve  # type: ignore
@@ -53,7 +54,7 @@ def env(name, default=None):
     """Access to environment variables
 
     Allows access to environment variables, falling back to config.vars (part
-    of c-lightning's `./configure` output), and finally falling back to a
+    of Core Lightning's `./configure` output), and finally falling back to a
     default value.
 
     """
@@ -628,22 +629,36 @@ class PrettyPrintingLightningRpc(LightningRpc):
             patch_json,
         )
         self.jsonschemas = jsonschemas
+        self.check_request_schemas = True
 
     def call(self, method, payload=None):
         id = self.next_id
+        schemas = self.jsonschemas.get(method)
         self.logger.debug(json.dumps({
             "id": id,
             "method": method,
             "params": payload
         }, indent=2))
+
+        # We only check payloads which are dicts, which is what we
+        # usually use: there are some cases which tests [] params,
+        # which we ignore.
+        if schemas and schemas[0] and isinstance(payload, dict) and self.check_request_schemas:
+            # fields which are None are explicitly removed, so do that now
+            testpayload = {}
+            for k, v in payload.items():
+                if v is not None:
+                    testpayload[k] = v
+            schemas[0].validate(testpayload)
+
         res = LightningRpc.call(self, method, payload)
         self.logger.debug(json.dumps({
             "id": id,
             "result": res
         }, indent=2))
 
-        if method in self.jsonschemas:
-            self.jsonschemas[method].validate(res)
+        if schemas and schemas[1]:
+            schemas[1].validate(res)
 
         return res
 
@@ -673,6 +688,8 @@ class LightningNode(object):
         socket_path = os.path.join(lightning_dir, TEST_NETWORK, "lightning-rpc").format(node_id)
         self.rpc = PrettyPrintingLightningRpc(socket_path, self.executor, jsonschemas=jsonschemas)
 
+        self.gossip_store = GossipStore(Path(lightning_dir, TEST_NETWORK, "gossip_store"))
+
         self.daemon = LightningD(
             lightning_dir, bitcoindproxy=bitcoind.get_proxy(),
             port=port, random_hsm=random_hsm, node_id=node_id
@@ -816,6 +833,7 @@ class LightningNode(object):
         self.info = self.rpc.getinfo()
         # This shortcut is sufficient for our simple tests.
         self.port = self.info['binding'][0]['port']
+        self.gossip_store.open()  # Reopen the gossip_store now that we should have one
         if wait_for_bitcoind_sync and not self.is_synced_with_bitcoin(self.info):
             wait_for(lambda: self.is_synced_with_bitcoin())
 
@@ -1138,6 +1156,39 @@ class LightningNode(object):
         except RpcError:
             return None
 
+    def dev_pay(self, bolt11, msatoshi=None, label=None, riskfactor=None,
+                maxfeepercent=None, retry_for=None,
+                maxdelay=None, exemptfee=None, use_shadow=True, exclude=[]):
+        """Wrapper for rpc.dev_pay which suppresses the request schema"""
+        # FIXME? dev options are not in schema
+        old_check = self.rpc.check_request_schemas
+        self.rpc.check_request_schemas = False
+        ret = self.rpc.dev_pay(bolt11, msatoshi, label, riskfactor,
+                               maxfeepercent, retry_for,
+                               maxdelay, exemptfee, use_shadow, exclude)
+        self.rpc.check_request_schemas = old_check
+        return ret
+
+    def dev_invoice(self, msatoshi, label, description, expiry=None, fallbacks=None, preimage=None, exposeprivatechannels=None, cltv=None, dev_routes=None):
+        """Wrapper for rpc.invoice() with dev-routes option"""
+        payload = {
+            "msatoshi": msatoshi,
+            "label": label,
+            "description": description,
+            "expiry": expiry,
+            "fallbacks": fallbacks,
+            "preimage": preimage,
+            "exposeprivatechannels": exposeprivatechannels,
+            "cltv": cltv,
+            "dev-routes": dev_routes,
+        }
+        # FIXME? dev options are not in schema
+        old_check = self.rpc.check_request_schemas
+        self.rpc.check_request_schemas = False
+        ret = self.rpc.call("invoice", payload)
+        self.rpc.check_request_schemas = old_check
+        return ret
+
 
 @contextmanager
 def flock(directory: Path):
@@ -1369,7 +1420,7 @@ class NodeFactory(object):
         # getpeers.
         if not fundchannel:
             for src, dst in connections:
-                dst.daemon.wait_for_log(r'{}-.*-chan#[0-9]*: Handed peer, entering loop'.format(src.info['id']))
+                dst.daemon.wait_for_log(r'{}-connectd: Handed peer, entering loop'.format(src.info['id']))
             return
 
         bitcoind = nodes[0].bitcoin
